package com.ibm.ccl.ua.generators.sitexml;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.StringReader;
import java.net.URL;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.jar.JarFile;
import java.util.zip.ZipEntry;

import javax.xml.transform.TransformerException;

import org.eclipse.core.runtime.Platform;
import org.eclipse.osgi.service.datalocation.Location;
import org.eclipse.update.configuration.IConfiguredSite;
import org.eclipse.update.configuration.ILocalSite;
import org.eclipse.update.core.IFeature;
import org.eclipse.update.core.IFeatureContentProvider;
import org.eclipse.update.core.INonPluginEntry;
import org.eclipse.update.core.ISite;
import org.eclipse.update.core.ISiteFeatureReference;
import org.eclipse.update.core.SiteManager;

public class SiteXMLHelper {

	private static final String CONFIG_DIR = "configuration"; //$NON-NLS-1$
	private static final String ECLIPSE = "eclipse"; //$NON-NLS-1$
	private static final String PRODUCT_SITE_MARKER = ".eclipseproduct"; //$NON-NLS-1$
	private static final String PRODUCT_SITE_ID = "id"; //$NON-NLS-1$
	private static final String PRODUCT_SITE_VERSION = "version"; //$NON-NLS-1$

	// constants: System property keys and/or configuration file elements
	private static final String PROP_USER_HOME = "user.home"; //$NON-NLS-1$
	
	static CategoryMappingHandler mappingHandler = new CategoryMappingHandler();
	static Hashtable categoryMappings = mappingHandler.getCategoryMappingsHash();

	public static boolean siteXMLExists()
	{
		boolean siteXMLExists = false;
		
		String configLocation = SiteXMLHelper.getConfigurationLocation();
		try
		{
					
			//Create folder and file entry in config location
			File configurationFile = new File(configLocation+"com.ibm.ccl.ua.generators.sitexml");
			if(!(configurationFile.exists()))
			{
				configurationFile.mkdir();
			}
			
			File newFile= new File(configLocation+ "com.ibm.ccl.ua.generators.sitexml"+File.separator+"site.xml");
			if(newFile.exists())
			{
				//First time remote config is being created
				siteXMLExists = true;
			}
			
			
		}
		catch(Exception e)
		{
			
		}
		
		return siteXMLExists;
	}
	
	public static void deleteSiteXML()
	{
		String configLocation = SiteXMLHelper.getConfigurationLocation();
		try
		{
					
			//Create folder and file entry in config location
			File configurationFile = new File(configLocation+"com.ibm.ccl.ua.generators.sitexml");
			if(!(configurationFile.exists()))
			{
				configurationFile.mkdir();
			}
			
			File newFile= new File(configLocation+ "com.ibm.ccl.ua.generators.sitexml"+File.separator+"site.xml");
			if(newFile.exists())
			{
				newFile.delete();
			}
			
			
		}
		catch(Exception e)
		{
			
		}
		
	}
	
	public static void createTestFile() throws IOException
	{
		String configLocation = SiteXMLHelper.getConfigurationLocation();
		File configurationFile = new File(configLocation+"com.ibm.ccl.ua.generators.sitexml");
		if(!(configurationFile.exists()))
		{
			configurationFile.mkdir();
		}
		File newFile2 = new File(configLocation+ "com.ibm.ccl.ua.generators.sitexml"+File.separator+"test.txt");
		newFile2.createNewFile();
		FileWriter outFile2 = new FileWriter(newFile2);
		PrintWriter out2 = new PrintWriter(outFile2);
		out2.write(SiteXMLHelper.computeDefaultConfigurationLocation2()+File.separator);
		out2.close();
	}
	public static void buildLocalSiteXML()
	{
		String configLocation = SiteXMLHelper.getConfigurationLocation();
		try
		{
					
			//Create folder and file entry in config location
			File configurationFile = new File(configLocation+"com.ibm.ccl.ua.generators.sitexml");
			if(!(configurationFile.exists()))
			{
				configurationFile.mkdir();
			}
			
			File newFile= new File(configLocation+ "com.ibm.ccl.ua.generators.sitexml"+File.separator+"site.xml");
			if(!(newFile.exists()))
			{
				newFile.createNewFile();
			}
			
			FileWriter outFile = new FileWriter(newFile);
			PrintWriter out = new PrintWriter(outFile);
			
			//Build SiteXML
			ArrayList featuresAdded = new ArrayList();
			ArrayList featuresToUpdate=new ArrayList();
			String currFeatureID;
			IFeature currFeature;
			int dupIndex=0;
			
			
			try
			{
				ILocalSite site = SiteManager.getLocalSite();

				IConfiguredSite[]sites = site.getCurrentConfiguration().getConfiguredSites();
				
				int j=0;
				
				for(j=0;j<sites.length;j++)
				{
					
					ISite uaSite = sites[j].getSite();
					ISiteFeatureReference [] uaFeatureReferences = uaSite.getFeatureReferences();
					
					int i;
					ISiteFeatureReference featureRef;
					IFeature siteFeature;
					
					for(i=0;i<uaFeatureReferences.length;i++)
					{
						featureRef = uaFeatureReferences[i];
						siteFeature=featureRef.getFeature();
						currFeatureID= siteFeature.getVersionedIdentifier().getIdentifier();
						
						
						if(featuresAdded.contains(currFeatureID)) 
						{
							//Feature already set to be added to the site.xml.  Need to check versions
							
							dupIndex=featuresAdded.indexOf(siteFeature.getVersionedIdentifier().getIdentifier());
							currFeature=(IFeature)featuresToUpdate.get(dupIndex);
							featuresToUpdate=VersionUtility.handleDuplicateFeature(featuresToUpdate, currFeature,siteFeature,dupIndex);
						
							
						}
						else
						{
							//New feature to add
							featuresToUpdate.add(siteFeature);
							featuresAdded.add(currFeatureID);
						}
						
					}
					
				}
				 
				//Generate Site XML
				String response = serialize(featuresToUpdate);
				out.write(response);
			}
			catch(Exception e)
			{
				
			}
			
			
			out.close();
			
		}
		catch(Exception e)
		{
			
		}
		
	}
	
	public static String getSiteXMLContents()
	{
		String siteXMLContents = "";
		byte[] siteXMLBytes;
		String configLocation = SiteXMLHelper.getConfigurationLocation();
		try
		{
					
			//Create folder and file entry in config location
			File configurationFile = new File(configLocation+"com.ibm.ccl.ua.generators.sitexml");
			if(!(configurationFile.exists()))
			{
				configurationFile.mkdir();
			}
			
			File newFile= new File(configLocation+ "com.ibm.ccl.ua.generators.sitexml"+File.separator+"site.xml");
			if(newFile.exists())
			{
				//First time remote config is being created
				FileInputStream fileInput = new FileInputStream(newFile);
				siteXMLBytes = new byte[fileInput.available()];
				fileInput.read(siteXMLBytes);
				siteXMLContents = new String(siteXMLBytes);
			}
			
			
		}
		catch(Exception e)
		{
			
		}
		
		return siteXMLContents;
	}
	public static String getConfigurationLocation()
	{
		String configLocation="";
		Location configLoc;
		
		try
		{
			
			configLocation = computeDefaultConfigurationLocation();
			configLocation = configLocation+File.separator;
			
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
				
		return configLocation;
	}
	
	private static String computeDefaultConfigurationLocation() {
		// 1) We store the config state relative to the 'eclipse' directory if possible
		// 2) If this directory is read-only 
		//    we store the state in <user.home>/.eclipse/<application-id>_<version> where <user.home> 
		//    is unique for each local user, and <application-id> is the one 
		//    defined in .eclipseproduct marker file. If .eclipseproduct does not
		//    exist, use "eclipse" as the application-id.
		
		/*Location configLoc = Platform.getInstallLocation();
		URL install = configLoc.getURL();
		// TODO a little dangerous here.  Basically we have to assume that it is a file URL.
		if (install.getProtocol().equals("file")) { //$NON-NLS-1$
			File installDir = new File(install.getFile());
			if (canWrite(installDir))
				return installDir.getAbsolutePath() + File.separator + CONFIG_DIR + File.separator;
		}
		// We can't write in the eclipse install dir so try for some place in the user's home dir
		//return computeDefaultUserAreaLocation(CONFIG_DIR);*/
		return "/usr/local/eclipse/eclipse34_update/ibm_help/eclipse/configuration/rsmhelp_v7r5m0/";
		
	}
	
	private static String computeDefaultConfigurationLocation2() {
		// 1) We store the config state relative to the 'eclipse' directory if possible
		// 2) If this directory is read-only 
		//    we store the state in <user.home>/.eclipse/<application-id>_<version> where <user.home> 
		//    is unique for each local user, and <application-id> is the one 
		//    defined in .eclipseproduct marker file. If .eclipseproduct does not
		//    exist, use "eclipse" as the application-id.
		
		Location configLoc = Platform.getConfigurationLocation();
		URL install = configLoc.getURL();
		// TODO a little dangerous here.  Basically we have to assume that it is a file URL.
		if (install.getProtocol().equals("file")) { //$NON-NLS-1$
			File installDir = new File(install.getFile());
			if (canWrite(installDir))
				return installDir.getAbsolutePath() + File.separator + CONFIG_DIR + File.separator;
		}
		// We can't write in the eclipse install dir so try for some place in the user's home dir
		return computeDefaultUserAreaLocation(CONFIG_DIR);
		
		
	}
	private static boolean canWrite(File installDir) {
		if (installDir.canWrite() == false)
			return false;

		if (!installDir.isDirectory())
			return false;

		File fileTest = null;
		try {
			// we use the .dll suffix to properly test on Vista virtual directories
			// on Vista you are not allowed to write executable files on virtual directories like "Program Files"
			fileTest = File.createTempFile("writtableArea", ".dll", installDir); //$NON-NLS-1$ //$NON-NLS-2$
		} catch (IOException e) {
			//If an exception occured while trying to create the file, it means that it is not writtable
			return false;
		} finally {
			if (fileTest != null)
				fileTest.delete();
		}
		return true;
	}
	
	private static String computeDefaultUserAreaLocation(String pathAppendage) {
		//    we store the state in <user.home>/.eclipse/<application-id>_<version> where <user.home> 
		//    is unique for each local user, and <application-id> is the one 
		//    defined in .eclipseproduct marker file. If .eclipseproduct does not
		//    exist, use "eclipse" as the application-id.
		
		Location installLoc = Platform.getInstallLocation();
		URL installURL = installLoc.getURL();
		if (installURL == null)
			return null;
		File installDir = new File(installURL.getFile());
		// compute an install dir hash to prevent configuration area collisions with other eclipse installs
		int hashCode;
		try {
			hashCode = installDir.getCanonicalPath().hashCode();
		} catch (IOException ioe) {
			// fall back to absolute path
			hashCode = installDir.getAbsolutePath().hashCode();
		}
		if (hashCode < 0)
			hashCode = -(hashCode);
		String installDirHash = String.valueOf(hashCode);

		String appName = "." + ECLIPSE; //$NON-NLS-1$
		File eclipseProduct = new File(installDir, PRODUCT_SITE_MARKER);
		if (eclipseProduct.exists()) {
			Properties props = new Properties();
			try {
				props.load(new FileInputStream(eclipseProduct));
				String appId = props.getProperty(PRODUCT_SITE_ID);
				if (appId == null || appId.trim().length() == 0)
					appId = ECLIPSE;
				String appVersion = props.getProperty(PRODUCT_SITE_VERSION);
				if (appVersion == null || appVersion.trim().length() == 0)
					appVersion = ""; //$NON-NLS-1$
				appName += File.separator + appId + "_" + appVersion + "_" + installDirHash; //$NON-NLS-1$ //$NON-NLS-2$
			} catch (IOException e) {
				// Do nothing if we get an exception.  We will default to a standard location 
				// in the user's home dir.
				// add the hash to help prevent collisions
				appName += File.separator + installDirHash;
			}
		} else {
			// add the hash to help prevent collisions
			appName += File.separator + installDirHash;
		}
		String userHome = System.getProperty(PROP_USER_HOME);
		return new File(userHome, appName + "/" + pathAppendage).getAbsolutePath(); //$NON-NLS-1$
	}

	private static String serialize(ArrayList featuresToUpdate) throws TransformerException {
		
		boolean updateFeature=false;
		String featureCategory;
		ArrayList featureCategories = new ArrayList();
		StringBuffer buf = new StringBuffer();
		buf.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"); //$NON-NLS-1$
		buf.append("<site>\n"); //$NON-NLS-1$
				
		String featureURL;
		//Build feature list
		for (int i = 0; i < featuresToUpdate.size(); ++i) {
			IFeature feature = (IFeature)featuresToUpdate.get(i);
			INonPluginEntry[] dataEntry =feature.getNonPluginEntries();	
			
			updateFeature=includeFeatureInUpdate(dataEntry, feature);
									
			featureCategory=getFeatureCategory(dataEntry,feature);
			
			if(!(featureCategories.contains(featureCategory)) && featureCategory!="")
			{
				featureCategories.add(featureCategory);			}
			
			if(updateFeature)
			{
				featureURL=filterFeatureURL(feature.getURL().toString());
				
				buf.append("<feature"); //$NON-NLS-1$
			
				buf.append("\n      url=\"" + featureURL + '"'); //$NON-NLS-1$
				buf.append("\n      id=\"" +  feature.getVersionedIdentifier().getIdentifier() + "\""); //$NON-NLS-1$
				
				if(featureCategory.equals(""))
				{
					buf.append("\n      version=\"" + feature.getVersionedIdentifier().getVersion()+ "\"/>\n"); //$NON-NLS-1$ //$NON-NLS-2$
				}
				else
				{
					buf.append("\n      version=\"" + feature.getVersionedIdentifier().getVersion()+ "\">\n"); //$NON-NLS-1$ //$NON-NLS-2$
				
					buf.append("\n     <category name=\"" + featureCategory+ "\"/>\n"); //$NON-NLS-1$ //$NON-NLS-2$
					
					buf.append("\n     </feature>\n"); //$NON-NLS-1$ //$NON-NLS-2$
				}
			}
		}
		
		//Add Category Defs
		int j=0;
		String currCategory;
		for(j=0;j<featureCategories.size();j++)
		{
			currCategory=(String)featureCategories.get(j);
			
			buf.append("\n     <category-def name=\"" + currCategory+ "\" label=\""+currCategory+"\"/>\n"); //$NON-NLS-1$ //$NON-NLS-2$
		}
		buf.append("</site>\n"); //$NON-NLS-1$
		return buf.toString();
	}

	public static boolean includeFeatureInUpdate(INonPluginEntry[] dataArray, IFeature currFeature)
	{
		boolean includeInUpdate=false;
		int i=0,j, charCount;
		INonPluginEntry currEntry;
		URL featureURL, propertiesURL;
		InputStream propertiesStream;
		InputStreamReader propertiesReader;
		char[] fileChars;
		String fileString;
		
		try
		{
			IFeatureContentProvider featureProvider = currFeature.getFeatureContentProvider();
		
			for(i=0;i<dataArray.length;i++)
			{
				currEntry=dataArray[i];
				
				
				if(currEntry.getIdentifier().contains("uadelivery.properties"))  //Found uapropertiues element
				{
					//Store the Contents of the UA Delivery properties file
					featureURL=featureProvider.getURL();
					
					if(!(featureURL.toString().endsWith(".jar")))
					{
						propertiesURL=new URL(featureURL.toString()+"/"+currEntry.getIdentifier());
						propertiesStream=propertiesURL.openStream();
					}
					else
					{
						//uadelivery.properties file in JAR
						
						String filePath = featureURL.toString();
						filePath = featureURL.getFile();
						JarFile featureJAR = new JarFile(filePath);
						
						ZipEntry uaDeliveryEntry = featureJAR.getEntry("uadelivery.properties");
						propertiesStream=featureJAR.getInputStream(uaDeliveryEntry);

						
					}
					
					charCount=propertiesStream.available();
					fileChars=new char[charCount];
					propertiesReader = new InputStreamReader(propertiesStream);
					propertiesReader.read(fileChars);						
					fileString = new String(fileChars);
									
					
					//Get the value of the includeInUpdate element
					
					StringReader myReader=new StringReader(fileString);
					int includeLocation;
					String includeEntry;
					includeLocation=fileString.indexOf("includeInUpdate");
					includeLocation=includeLocation+16;		
					includeEntry=fileString.substring(includeLocation,includeLocation+4);
					
					includeEntry=includeEntry.toLowerCase();
					//If includeInUpdate = true, then set include to true
					if(includeEntry.equals("true"))
					{
						includeInUpdate=true;
					}		

					
				}
			
			}
		
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		
		return includeInUpdate;
	}
	
	public static String getFeatureCategory(INonPluginEntry[] dataArray, IFeature currFeature)
	{
		String featureCategory="";
		int i=0,j, charCount;
		INonPluginEntry currEntry;
		URL featureURL, propertiesURL=null;
		InputStream propertiesStream=null;
		InputStreamReader propertiesReader;
		char[] fileChars;
		String fileString;
		
		try
		{
			IFeatureContentProvider featureProvider = currFeature.getFeatureContentProvider();
		
			for(i=0;i<dataArray.length;i++)
			{
				currEntry=dataArray[i];
				
				if(currEntry.getIdentifier().contains("uadelivery.properties"))  //Found uapropertiues element
				{
					//Store the Contents of the UA Delivery properties file
					featureURL=featureProvider.getURL();
					
					if(!(featureURL.toString().endsWith(".jar")))
					{
						propertiesURL=new URL(featureURL.toString()+"/"+currEntry.getIdentifier());
						propertiesStream=propertiesURL.openStream();
					}
					else
					{
						//uadelivery.properties file in JAR
						
						String filePath = featureURL.toString();
						filePath = featureURL.getFile();
						JarFile featureJAR = new JarFile(filePath);
						
						ZipEntry uaDeliveryEntry = featureJAR.getEntry("uadelivery.properties");
						propertiesStream=featureJAR.getInputStream(uaDeliveryEntry);

						
					}
					
					charCount=propertiesStream.available();
					fileChars=new char[charCount];
					propertiesReader = new InputStreamReader(propertiesStream);
					propertiesReader.read(fileChars);						
					fileString = new String(fileChars);
									
					
					//Get the value of the includeInUpdate element
					
					StringReader myReader=new StringReader(fileString);
					int categoryLocation;
					String categoryEntry;
					categoryLocation=fileString.indexOf("category");
					
					if(categoryLocation!=-1)
					{
						categoryLocation=categoryLocation+9;		
						categoryEntry=fileString.substring(categoryLocation,fileString.length());
					
						StringTokenizer categoryTokenizer = new StringTokenizer(categoryEntry,"\n");
						categoryEntry=categoryTokenizer.nextToken();
						featureCategory=categoryEntry;
						
						//Check if category is mapped to another category
						String currMapCategory = (String)categoryMappings.get(featureCategory);
						
						if(currMapCategory!=null)
						{
							//Category has map entry in the hash table and needs to be mapped
							featureCategory=currMapCategory;
						}
						
					}
					else
					{
						featureCategory="";
					}
				}
			
			}
		
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		
		return filterFeatureCategory(featureCategory);
	}
	
	
	public static String filterFeatureURL(String featureURL)
	{
		String filteredURL;
			
		int featureIndex=featureURL.lastIndexOf("features/");
		filteredURL=featureURL.substring(featureIndex);
		
		return filteredURL;
	}
	
	public String filterJARPath(String jarPath)
	{
		String newJARPath;
		
		newJARPath=jarPath.replace("/","\\");
		
		newJARPath=jarPath.substring(6);
		
		return newJARPath;
	}
	
	public static String filterFeatureCategory(String currCategory)
	{
		String filteredCategory;
		filteredCategory=currCategory.replace("/r", "");
		return filteredCategory.trim();
	}
}
