/*******************************************************************************
 * Copyright (c) 2004, 2005 IBM Corporation.
 * All rights reserved. 
 *******************************************************************************/
package com.ibm.gallery.common.wizards;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExecutableExtension;
import org.eclipse.core.runtime.IStatus;

import org.eclipse.swt.widgets.Display;

import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;

import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.actions.WorkspaceModifyDelegatingOperation;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.wizards.newresource.BasicNewProjectResourceWizard;
import org.eclipse.ui.wizards.newresource.BasicNewResourceWizard;

import org.eclipse.ui.dialogs.IOverwriteQuery;
import org.eclipse.jface.dialogs.IDialogConstants;

import com.ibm.ccl.ua.wizards.*;

public class ExampleProjectCreationWizard extends Wizard implements INewWizard, IExecutableExtension {

	private ExampleProjectCreationWizardPage[] pages;
	private IConfigurationElement configElement;
	private ArrayList newPages= new ArrayList();

	public ExampleProjectCreationWizard() {
		super();
	}
	
	private void initializeDefaultPageImageDescriptor() {
		if (configElement != null) {
			String banner= configElement.getAttribute("banner"); //$NON-NLS-1$
			if (banner != null) {
				ImageDescriptor desc = this.getImageDescriptor(banner);
				setDefaultPageImageDescriptor(desc);
			// use the default icon in gallery common	
			} else {
				ImageDescriptor desc = this.getImageDescriptor("images/import_wiz.gif");
				setDefaultPageImageDescriptor(desc);
			}
		}
	}
	
	protected ImageDescriptor getImageDescriptor(String banner) {
		try {
			ImageDescriptor image = Activator.imageDescriptorFromPlugin("com.ibm.gallery.common", banner);
			return image;
		} catch (Exception e) {
			return ImageDescriptor.getMissingImageDescriptor();
		}
	}
	
	/*
	 * @see Wizard#addPages
	 */	
	public void addPages() {
		super.addPages();
		
		if(configElement!=null)
		{
			IConfigurationElement[] children = configElement.getChildren("projectsetup"); //$NON-NLS-1$
			if (children == null || children.length == 0) {
				Activator.logWarning("descriptor must contain one ore more projectsetup tags"); //$NON-NLS-1$
				return;
			}
		
			pages = new ExampleProjectCreationWizardPage[children.length];
		
			for (int i= 0; i < children.length; i++) {
				pages[i]= new ExampleProjectCreationWizardPage(i, children[i]);
				addPage(pages[i]);
			}
		}
		else
		{
			//Pages already have been added
			pages = new ExampleProjectCreationWizardPage[newPages.size()];
			
			for(int j=0;j<newPages.size();j++)
			{
				ExampleProjectCreationWizardPage currPage = (ExampleProjectCreationWizardPage)newPages.get(j);
				pages[j]=currPage;
				addPage(pages[j]);
				
			}
			
			
			
			
			//pages[0]= new ExampleProjectCreationWizardPage(currPage., children[i]);
			//addPage(pages[i]);
			
		}
	}
	
	/*
	 * @see Wizard#performFinish
	 */		
	public boolean performFinish() {
		// v601 version
		//ExampleProjectCreationOperation runnable = new ExampleProjectCreationOperation(pages, new ImportOverwriteQuery());
		ExampleProjectCreationOperation runnable = new ExampleProjectCreationOperation(this.pages);
		
		IRunnableWithProgress op = new WorkspaceModifyDelegatingOperation(runnable);
		try {
			getContainer().run(false, true, op);
		} catch (InvocationTargetException e) {
			handleException(e.getTargetException());
			return false;
		} catch  (InterruptedException e) {
			Activator.logWarning("ExampleProjectCreationWizard performFinish() " + e);
			e.printStackTrace();
			return false;
		}
		
		if(configElement!=null)
		{
			BasicNewProjectResourceWizard.updatePerspective(configElement);
		}
		else
		{
			//Get the wizard page which has the final perspective value, and call the update perspective method
			ExampleProjectCreationWizardPage importWizardPage = (ExampleProjectCreationWizardPage)newPages.get(0);
			PerspectiveSwitchUtility.updatePerspective(importWizardPage.finalPerspective);
		}
		IResource res = runnable.getElementToOpen();
		if (res != null) {
			openResource(res);
		}
		return true;
	}
	
	private void handleException(Throwable target) {
		String title= ExampleProjectMessages.getString("ExampleProjectCreationWizard.op_error.title"); //$NON-NLS-1$
		String message= ExampleProjectMessages.getString("ExampleProjectCreationWizard.op_error.message"); //$NON-NLS-1$
		if (target instanceof CoreException) {
			IStatus status= ((CoreException)target).getStatus();
			ErrorDialog.openError(getShell(), title, message, status);
			Activator.log(status);
		} else {
			MessageDialog.openError(getShell(), title, target.getMessage());
			Activator.log(target);
		}
	}
	
	private void openResource(final IResource resource) {
		if (resource.getType() != IResource.FILE) {
			return;
		}
		
		IWorkbenchWindow window= Activator.getDefault().getWorkbench().getActiveWorkbenchWindow();
		if (window == null) {
			return;
		}
		final IWorkbenchPage activePage= window.getActivePage();
		if (activePage != null) {
			final Display display= getShell().getDisplay();
			display.asyncExec(new Runnable() {
				public void run() {
					try {
						IDE.openEditor(activePage, (IFile)resource, true);
					} catch (PartInitException e) {
						Activator.log(e);
					}
				}
			});
			BasicNewResourceWizard.selectAndReveal(resource, activePage.getWorkbenchWindow());
		}
	}	
		
	/**
	 * Stores the configuration element for the wizard.  The config element will be used
	 * in <code>performFinish</code> to set the result perspective.
	 */
	public void setInitializationData(IConfigurationElement cfig, String propertyName, Object data) {
		configElement= cfig;
		
		initializeDefaultPageImageDescriptor();

		String title= configElement.getAttribute("name"); //$NON-NLS-1$
		if (title != null) {
			setWindowTitle(title);	
		}
	}
	
	// overwrite dialog
	//
	private class ImportOverwriteQuery implements IOverwriteQuery {
		public String queryOverwrite(String file) {
			String[] returnCodes= { YES, NO, ALL, CANCEL};
			int returnVal= openDialog(file);
			return returnVal < 0 ? CANCEL : returnCodes[returnVal];
		}	
		
		private int openDialog(final String file) {
			final int[] result= { IDialogConstants.CANCEL_ID };
			getShell().getDisplay().syncExec(new Runnable() {
				public void run() {
					String title= ExampleProjectMessages.getString("ExampleProjectCreationWizard.overwritequery.title"); //$NON-NLS-1$
					String msg= ExampleProjectMessages.getFormattedString("ExampleProjectCreationWizard.overwritequery.message", file); //$NON-NLS-1$
					String[] options= {IDialogConstants.YES_LABEL, IDialogConstants.NO_LABEL, IDialogConstants.YES_TO_ALL_LABEL, IDialogConstants.CANCEL_LABEL};
					MessageDialog dialog= new MessageDialog(getShell(), title, null, msg, MessageDialog.QUESTION, options, 0);
					result[0]= dialog.open();
				}
			});
			return result[0];
		}
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench, org.eclipse.jface.viewers.IStructuredSelection)
	 */
	public void init(IWorkbench workbench, IStructuredSelection selection) {
	}	
	
	public void addNewPage(ExampleProjectCreationWizardPage newPage)
	{
		newPages.add(newPage);
	}
}
